@@ -1,15 +1,14 @@
 // modules
-mod lazy;
 mod parser;
 mod query;
 
 // re-export
-pub use lazy::LazyLoadingComments;
-pub use parser::{Comment, Story};
+pub use parser::{Comment, CommentState, Story};
 pub use query::StoryNumericFilters;
 
 use crate::prelude::*;
 use parser::*;
+use rayon::prelude::*;
 
 const HN_ALGOLIA_PREFIX: &str = "https://hn.algolia.com/api/v1";
 const HN_OFFICIAL_PREFIX: &str = "https://hacker-news.firebaseio.com/v0";
@@ -19,16 +18,30 @@ pub const HN_HOST_URL: &str = "https://news.ycombinator.com";
 
 static CLIENT: once_cell::sync::OnceCell<HNClient> = once_cell::sync::OnceCell::new();
 
+pub type CommentSender = crossbeam_channel::Sender<Vec<Comment>>;
+pub type CommentReceiver = crossbeam_channel::Receiver<Vec<Comment>>;
+
 /// HNClient is a HTTP client to communicate with Hacker News APIs.
 #[derive(Clone)]
 pub struct HNClient {
     client: ureq::Agent,
 }
 
+macro_rules! log {
+    ($e:expr, $desc:expr) => {{
+        let time = std::time::SystemTime::now();
+        let result = $e;
+        if let Ok(elapsed) = time.elapsed() {
+            info!("{} took {}ms", $desc, elapsed.as_millis());
+        }
+        result
+    }};
+}
+
 impl HNClient {
     /// Create a new Hacker News Client
     pub fn new() -> Result<HNClient> {
-        let timeout = get_config().client.client_timeout;
+        let timeout = config::get_config().client.client_timeout;
         Ok(HNClient {
             client: ureq::AgentBuilder::new()
                 .timeout(std::time::Duration::from_secs(timeout))
@@ -43,53 +56,86 @@ impl HNClient {
         T: serde::de::DeserializeOwned,
     {
         let request_url = format!("{}/items/{}", HN_ALGOLIA_PREFIX, id);
-        let time = std::time::SystemTime::now();
-        let item = self.client.get(&request_url).call()?.into_json::<T>()?;
-        if let Ok(elapsed) = time.elapsed() {
-            info!("get item id={} took {}ms", id, elapsed.as_millis());
-        }
+        let item = log!(
+            self.client.get(&request_url).call()?.into_json::<T>()?,
+            format!("get HN item (id={}) using {}", id, request_url)
+        );
         Ok(item)
     }
 
-    /// Get all comments from a story with a given id.
-    pub fn get_comments_from_story(
-        &self,
-        story_id: u32,
-        focus_top_comment_id: u32,
-    ) -> Result<lazy::LazyLoadingComments> {
+    /// Lazily load a story's comments
+    pub fn lazy_load_story_comments(&self, story_id: u32) -> Result<CommentReceiver> {
         let request_url = format!("{}/item/{}.json", HN_OFFICIAL_PREFIX, story_id);
-        let mut ids = self
-            .client
-            .get(&request_url)
-            .call()?
-            .into_json::<HNStoryResponse>()?
-            .kids;
-        if let Some(pos) = ids.iter().position(|id| *id == focus_top_comment_id) {
-            // move `pos` to the beginning of the list.
-            ids.remove(pos);
-            ids.insert(0, focus_top_comment_id);
-        };
+        let mut ids = log!(
+            self.client
+                .get(&request_url)
+                .call()?
+                .into_json::<HNStoryResponse>()?
+                .kids,
+            format!("get story (id={}) using {}", story_id, request_url)
+        );
 
-        let mut comments = lazy::LazyLoadingComments::new(self.clone(), ids);
+        let (sender, receiver) = crossbeam_channel::bounded(32);
+        let client = self.clone();
 
-        let cfg = &(get_config().client.lazy_loading_comments);
-        comments.drain(cfg.num_comments_init, true);
-        comments.drain(cfg.num_comments_after, false);
-        Ok(comments)
+        // loads first 5 comments to ensure the corresponding `CommentView` has data to render
+        Self::load_comments(&client, &sender, &mut ids, 5)?;
+        std::thread::spawn(move || {
+            let sleep_dur = std::time::Duration::from_millis(1000);
+            while !ids.is_empty() {
+                if let Err(err) = Self::load_comments(&client, &sender, &mut ids, 5) {
+                    warn!("encountered an error when loading comments: {}", err);
+                    break;
+                }
+                std::thread::sleep(sleep_dur);
+            }
+        });
+        Ok(receiver)
+    }
+
+    /// Load the first `size` comments from a list of comment IDs.
+    fn load_comments(
+        client: &HNClient,
+        sender: &CommentSender,
+        ids: &mut Vec<u32>,
+        size: usize,
+    ) -> Result<()> {
+        let size = std::cmp::min(ids.len(), size);
+        if size == 0 {
+            return Ok(());
+        }
+
+        let responses = ids
+            .drain(0..size)
+            .collect::<Vec<_>>()
+            .into_par_iter()
+            .map(|id| match client.get_item_from_id::<CommentResponse>(id) {
+                Ok(response) => Some(response),
+                Err(err) => {
+                    warn!("failed to get comment (id={}): {}", id, err);
+                    None
+                }
+            })
+            .flatten()
+            .collect::<Vec<_>>();
+
+        for response in responses {
+            sender.send(response.into())?;
+        }
+
+        Ok(())
     }
 
     /// Get a story based on its id
     pub fn get_story_from_story_id(&self, id: u32) -> Result<Story> {
         let request_url = format!("{}/search?tags=story,story_{}", HN_ALGOLIA_PREFIX, id);
-        let time = std::time::SystemTime::now();
-        let response = self
-            .client
-            .get(&request_url)
-            .call()?
-            .into_json::<StoriesResponse>()?;
-        if let Ok(elapsed) = time.elapsed() {
-            info!("get story (id={}) took {}ms", id, elapsed.as_millis());
-        }
+        let response = log!(
+            self.client
+                .get(&request_url)
+                .call()?
+                .into_json::<StoriesResponse>()?,
+            format!("get story (id={}) using {}", id, request_url)
+        );
 
         let stories: Vec<Story> = response.into();
         Ok(stories.first().unwrap().clone())
@@ -102,7 +148,7 @@ impl HNClient {
         by_date: bool,
         page: usize,
     ) -> Result<Vec<Story>> {
-        let search_story_limit = get_config().client.story_limit.search;
+        let search_story_limit = config::get_config().client.story_limit.search;
         let request_url = format!(
             "{}/{}?{}&hitsPerPage={}&page={}",
             HN_ALGOLIA_PREFIX,
@@ -111,21 +157,17 @@ impl HNClient {
             search_story_limit,
             page
         );
-        let time = std::time::SystemTime::now();
-        let response = self
-            .client
-            .get(&request_url)
-            .query("query", query)
-            .call()?
-            .into_json::<StoriesResponse>()?;
-        if let Ok(elapsed) = time.elapsed() {
-            info!(
-                "get matched stories with query {} (by_date={}) took {}ms",
-                query,
-                by_date,
-                elapsed.as_millis()
-            );
-        }
+        let response = log!(
+            self.client
+                .get(&request_url)
+                .query("query", query)
+                .call()?
+                .into_json::<StoriesResponse>()?,
+            format!(
+                "get matched stories with query {} (by_date={}, page={}) using {}",
+                query, by_date, page, request_url
+            )
+        );
 
         Ok(response.into())
     }
@@ -164,19 +206,13 @@ impl HNClient {
         numeric_filters: query::StoryNumericFilters,
     ) -> Result<Vec<Story>> {
         let request_url = format!("{}/topstories.json", HN_OFFICIAL_PREFIX);
-        let time = std::time::SystemTime::now();
-        let stories = self
-            .client
-            .get(&request_url)
-            .call()?
-            .into_json::<Vec<u32>>()?;
-        if let Ok(elapsed) = time.elapsed() {
-            info!(
-                "get front_page story ids using {} took {}ms",
-                request_url,
-                elapsed.as_millis()
-            );
-        }
+        let stories = log!(
+            self.client
+                .get(&request_url)
+                .call()?
+                .into_json::<Vec<u32>>()?,
+            format!("get front page stories using {}", request_url)
+        );
 
         let start_id = story_limit * page;
         if start_id >= stories.len() {
@@ -197,18 +233,16 @@ impl HNClient {
             numeric_filters.query(),
         );
 
-        let response = self
-            .client
-            .get(&request_url)
-            .call()?
-            .into_json::<StoriesResponse>()?;
-        if let Ok(elapsed) = time.elapsed() {
-            info!(
-                "get stories (tag=front_page, by_date=false, page={}) took {}ms",
-                page,
-                elapsed.as_millis()
-            );
-        }
+        let response = log!(
+            self.client
+                .get(&request_url)
+                .call()?
+                .into_json::<StoriesResponse>()?,
+            format!(
+                "get stories (tag=front_page, by_date=false, page={}) using {}",
+                page, request_url
+            )
+        );
 
         Ok(self.reoder_front_page_stories(response.into(), ids))
     }
@@ -221,7 +255,10 @@ impl HNClient {
         page: usize,
         numeric_filters: query::StoryNumericFilters,
     ) -> Result<Vec<Story>> {
-        let story_limit = get_config().client.story_limit.get_story_limit_by_tag(tag);
+        let story_limit = config::get_config()
+            .client
+            .story_limit
+            .get_story_limit_by_tag(tag);
 
         if tag == "front_page" {
             return self.get_front_page_stories(story_limit, page, numeric_filters);
@@ -236,21 +273,20 @@ impl HNClient {
             numeric_filters.query(),
         );
 
-        let time = std::time::SystemTime::now();
-        let response = self
-            .client
-            .get(&request_url)
-            .call()?
-            .into_json::<StoriesResponse>()?;
-        if let Ok(elapsed) = time.elapsed() {
-            info!(
-                "get stories (tag={}, by_date={}, page={}) took {}ms",
+        let response = log!(
+            self.client
+                .get(&request_url)
+                .call()?
+                .into_json::<StoriesResponse>()?,
+            format!(
+                "get stories (tag={}, by_date={}, page={}, numeric_filters={}) using {}",
                 tag,
                 by_date,
                 page,
-                elapsed.as_millis()
-            );
-        }
+                numeric_filters.query(),
+                request_url
+            )
+        );
 
         Ok(response.into())
     }
@@ -261,5 +297,5 @@ pub fn init_client() -> &'static HNClient {
     CLIENT.set(client).unwrap_or_else(|_| {
         panic!("failed to set up the application's HackerNews Client");
     });
-    &CLIENT.get().unwrap()
+    CLIENT.get().unwrap()
 }
